from Martel import Re, Str, Rep, RepN, Group, Opt, Integer, Rep1

header1 = Str("""\
 
                P R O T E I N  S E Q U E N C E  D A T A B A S E
                             of PIR-International
 
""")
header2 = Re("""\
                         Section (?P<section_id>\d+). (?P<section_title>.*)
                   Release (?P<release>(?P<major_version>\d+)\.(?P<minor_version>\d+)),  (?P<date>.*)
                        (?P<num_sequences>\d+) sequences, (?P<num_residues>\d+) residues
 
""")
header3 = Str("""\
                      Protein Information Resource (PIR)*
                    National Biomedical Research Foundation
                          3900 Reservoir Road, N.W.,
                          Washington, DC  20007, USA
 
   Japan International Protein           Munich Information Center for
   Information Database (JIPID)             Protein Sequences (MIPS)
         Amakubo 1-16-1          GSF-Forschungszentrum f. Umwelt und Gesundheit
    Tsukuba 305-0005, Japan            am Max-Planck-Instut f. Biochemie
                                  Am Klopferspitz 18, D-82152 Martinsried, FRG
 
 
""")
header4 = Str("""\
   This database may be redistributed without prior consent, provided that
   this notice be given to each user and that the words "Derived from" shall
   precede this notice if the database has been altered by the redistributor.
 
                       *PIR is a registered mark of NBRF.
 
 
\\\
""")
header =  Group("header", header1 + header2 + header3 + header4)

entry = Group("entry",
              Str("ENTRY            ") + \
              Group("id", Re(r"\w+")) + \
              Re(r"\s+") + \
              Str("#type ") + \
              Group("type",
                    Str("complete") |
                    Str("fragment") |
                    Str("fragments")) + \
              Str("\n")
              )

def make_block(identifier, name):
    return Group(name + "_block",
                 Group(name + "_line",
                       Str(identifier + " " * (17-len(identifier))) + \
                       Group(name, Re(".*")) + Str("\n")) + \
                 Rep(Group(name + "_line",
                           Str(" " * 17) + Group(name, Re(".*")) + Str("\n")))
                 )

title = make_block("TITLE", "title")


alternate_names = None

contains = None

space = Str(" ") | \
        Str("\n" + " "*17 + "  ")
word = Re(r'[!"$-~]+')

formal_name = Group("formal_name",
                    Str("#formal_name") + \
                    Rep(space + Group("word", word)))
common_name = Group("common_name",
                    Str("#common_name") + \
                    Rep(space + Group("word", word)))
organism = Str("ORGANISM         ") + \
           formal_name + \
           Opt(space + common_name) + \
           Str("\n")

date_format = Group("date",
                    Group("day", Re(r"\d+")) + Str("-") + \
                    Group("month", Re(r"\w+")) + Str("-") + \
                    Group("year", Re(r"\d+")))

date = Str("DATE             ") + \
       Group("date_added", date_format) + \
       Str(" ") + \
       Group("sequence_revision",
             Str("#sequence_revision ") + \
             date_format) + \
       Str(" ") + \
       Group("text_change",
             Str("#text_change") + \
             Str("\n                   ") + \
             date_format) + \
       Str("\n")

accessions = Str("ACCESSIONS       ") + \
             Group("accession", Re(r"\w+")) + \
             Rep( Str("; ") + Group("accession", Re(r"\w+"))) + \
             Str("\n")

reference_number = Re("REFERENCE        (?P<reference_number>.*)\n")

journal_subitem = Group("journal_subitem",
                        Str("   #journal      ") + \
                        Group("journal_abbreviation", Re(r"[^(\n]+")) + \
                        Re(r"\((?P<year>\d+)\)") + \
                        Str(" ") + \
                        Opt(Re(r"(?P<volume>\d+)?")) + \
                        Opt(Re(r"(\[(?P<issue>\d+)\])?")) + \
                        Str(":") + \
                        Group("pages",
                              Re(r"(?P<start_page>\d+)-(?P<end_page>\d+)")) + \
                        Str("\n"))

                        
submission_subitem = Group("submission_subitem",
                           Str("   #submission   ") + \
                           Str("submitted to the ") + \
                           Group("submission_database", Re("[^,\n]*")) + \
                           Str(", ") + \
                           Group("submission_date", Re(
    r"(?P<submission_month>\w+) (?P<submission_year>\d+)")) + \
                           Str("\n"))


# Need examples for these
book_subitem = None
citation_subitem = None

title_subitem = Group("title_subitem",
                      Str("   #title        ") + \
                      Group("title", Re(".*")) + \
                      Str("\n") + \
                      Rep(Str("                   ") + \
                          Group("title", Re(".*")) + \
                          Str("\n")))

description_subitem = Group("description_subitem",
                            Str("   #description  ") + \
                            Group("description", Re(".*")) + \
                            Str("\n") + \
                            Rep(Str("                   ") + \
                                Group("description", Re(".*")) + \
                                Str("\n")))

reference_title_subitem = Group("reference_title_subitem",
                                title_subitem | description_subitem)

# cross-references to medline
xref_subitem = Group("cross_reference_subitem",
                     Str("   #cross-references ") + \
                     Group("xref",
                           Group("xref_database", Str("MUID")) + \
                           Str(":") + \
                           Group("xref_id", Re(r"\d+"))) + \
                     Str("\n"))

# Need an example
contents_subitem = None

note_subitem = None

accession_start = Str("   #accession    ") + \
                         Group("accession_number", Re(".*")) + \
                         Str("\n")
accession_status = Str("      ##status        ") + \
                   Group("status", Re("\w*")) + \
                   Str("\n")
accession_molecule_type = Str("      ##molecule_type ") + \
                          Group("molecule_type",
                   Re("DNA|RNA|genomic RNA|mRNA|nucleic acid|protein")) + \
                          Str("\n")
accession_residues = Str("      ##residues      ") + \
                     Re("(?P<residue_data>[^#\n]*)") + \
                     Rep(Str("\n                        ") + \
                         Re("(?P<residue_data>[^#\n]*)")) + \
                     Re("##label (?P<residue_label>\w+)") + \
                     Str("\n")

# The id can be "", at least, so says T50374 in pir3 which has an
# "SPDB:"
xref_database = Group("xref",
                      Re("(?P<xref_database>[^:\n]+):" \
                         "(?P<xref_id>[^:;\n]*)"))
accession_xrefs = Str("      ##cross-references ") + \
                  xref_database + \
                  Rep(Str("; ") + xref_database) + \
                  Rep(Str(";\n                         ") + \
                      Rep(xref_database) + \
                      Rep(Str("; ") + xref_database)) + \
                  Str("\n")

                      

accession_genetics = None
accession_note = None

accession_subitem = Group("accession_subitem",
                          accession_start + \
                          accession_status + \
                          Opt(accession_molecule_type) + \
                          accession_residues + \
                          Opt(accession_xrefs)
                          )
                          
                                 
reference_authors = Str("   ") + \
                    Group("authors",
                          Str("#authors      ") + \
                          Group("authors_data", Re(".*")) + \
                          Rep(Str("\n                   ") + \
                              Group("authors_data", Re(".*"))) + \
                          Str("\n"))
reference_submission = Str("   ") + \
                       Group("submission",
                             Str("#submission   ",))

reference = reference_number + \
            reference_authors + \
            ((journal_subitem + title_subitem) | \
             (submission_subitem + Opt(description_subitem)) ) + \
            Opt(xref_subitem) + \
            accession_subitem 
            
            

comment = None

genetics = None

#?complex = None

#?function = None

classification = None

keywords = None

feature = None

summary = Str("SUMMARY          ") + \
          Str("#length ") + Integer("length") + \
          Opt(Str("  #molecular-weight ") + Integer("molecular_weight")) + \
          Str("  #checksum ") + Integer("checksum") + \
          Str("\n")


sequence_header = Str("SEQUENCE\n") + \
                  Re(r"        [^\n]*\n")
sequence_data = Rep1(
    Re("   (?P<start_pos>.....)([()=/., ](?P<sequence>[A-Z]))+[()=/., ]?\n")
    )

sequence = sequence_header + Rep(Group("sequence_data", sequence_data))

end_of_entry = Str("///\n")

record = entry + \
         title + \
         organism + \
         date + \
         accessions + \
         reference + \
         summary + \
         sequence + \
         end_of_entry
